<?php
/**
 * Copyright (с) Cloud Linux GmbH & Cloud Linux Software, Inc 2010-2022 All Rights Reserved
 *
 * Licensed under CLOUD LINUX LICENSE AGREEMENT
 * https://www.cloudlinux.com/legal/
 */

namespace CloudLinux\SmartAdvice\App\Advice;

use CloudLinux\SmartAdvice\App\Option;
use CloudLinux\SmartAdvice\App\Notice\Model as NoticeModel;

/**
 * Advice manager
 */
class Manager {
	/**
	 * Api.
	 *
	 * @var Api
	 */
	private $api;

	/**
	 * Option.
	 *
	 * @var Option
	 */
	private $option;

	/**
	 * Site home URL.
	 *
	 * @var string
	 */
	private $home = '';

	/**
	 * Site domain.
	 *
	 * @var string
	 */
	private $domain = '';

	/**
	 * WordPress path.
	 *
	 * @var string
	 */
	private $website = '';

	/**
	 * Advices.
	 *
	 * @var array<Model>
	 */
	protected $advices;

	/**
	 * Constructor.
	 *
	 * @param Api    $api api.
	 * @param Option $option api.
	 */
	public function __construct( Api $api, Option $option ) {
		$this->api     = $api;
		$this->option  = $option;
		$this->advices = $this->cache();
		$this->env();

		add_action( 'cl_smart_advice_sync', array( $this, 'sync' ), 10, 1 );
		add_action( 'cl_smart_advice_sync_statuses', array( $this, 'syncStatuses' ), 10, 1 );
		add_action( 'cl_smart_advice_status', array( $this, 'status' ), 10, 2 );
		add_action( 'cl_smart_advice_apply', array( $this, 'apply' ), 10, 2 );
		add_action( 'cl_smart_advice_apply_type', array( $this, 'applyType' ), 10, 2 );
		add_action( 'cl_smart_advice_rollback', array( $this, 'rollback' ), 10, 3 );
		add_action( 'cl_smart_advice_subscription', array( $this, 'subscription' ), 10, 1 );
	}

	/**
	 * Evn info.
	 *
	 * @return void
	 */
	private function env() {
		if ( defined( 'WP_HOME' ) ) {
			$home = (string) WP_HOME;
		} else {
			$home = (string) get_option( 'home' );
		}

		$this->home = $home;

		$parse = wp_parse_url( $home );

		if ( ! is_array( $parse ) ) {
			$parse = array();
		}

		$domain = array_key_exists( 'host', $parse ) ? $parse['host'] : '';
		$path   = array_key_exists( 'path', $parse ) ? $parse['path'] : '';

		$this->domain  = $domain;
		$this->website = ! empty( $path ) ? $path : '/';
	}

	/**
	 * Domain.
	 *
	 * @return string
	 */
	public function domain() {
		return $this->domain;
	}

	/**
	 * Website.
	 *
	 * @return string
	 */
	public function website() {
		return $this->website;
	}

	/**
	 * Home URL
	 *
	 * @return string
	 */
	public function home() {
		return $this->home;
	}

	/**
	 * Api interface.
	 *
	 * @return Api
	 */
	private function api() {
		return $this->api;
	}

	/**
	 * Advices array.
	 *
	 * @return array<Model>
	 */
	public function advices() {
		$advices = $this->advices;

		uasort(
			$advices,
			function ( $a, $b ) {
				if ( 'review' === $a->status ) {
					return - 1;
				}

				return 1;
			}
		);

		return $advices;
	}

	/**
	 * Check domain and WordPress path.
	 *
	 * @param array $item data.
	 *
	 * @return boolean
	 */
	private function checkMetadata( $item ) {
		if ( strtolower( $this->domain() ) !== strtolower( $item['metadata']['domain'] ) ) {
			return false;
		}

		if ( strtolower( $this->website() ) !== strtolower( $item['metadata']['website'] ) ) {
			return false;
		}

		return true;
	}

	/**
	 * Check advice.
	 *
	 * @param array $advice advice.
	 *
	 * @return boolean
	 */
	private function checkAdvice( $advice ) {
		if ( 'outdated' === $advice['status'] ) {
			return false;
		}

		return true;
	}

	/**
	 * Add or update advice.
	 *
	 * @param array $advice data.
	 *
	 * @return self
	 */
	private function set( $advice ) {
		if ( array_key_exists( $advice['id'], $this->advices() ) ) {
			$model = $this->advices()[ $advice['id'] ];
		} else {
			$model = new Model();
		}

		$model->fill( $advice );

		$this->advices[ $advice['id'] ] = $model;

		return $this;
	}

	/**
	 * Sync advice.
	 *
	 * @param array $response advices.
	 *
	 * @return void
	 */
	public function sync( $response ) {
		if ( is_array( $response ) && array_key_exists( 'data', $response ) ) {
			$this->setAdvices( $response );
			$this->notifyNew();
			$this->save();
		} else {
			add_filter( 'cl_smart_advice_sync_failed', '__return_true' );
		}
	}

	/**
	 * Set advices.
	 *
	 * @param mixed $response advices.
	 *
	 * @return void
	 */
	public function setAdvices( $response ) {
		$this->advices = array();

		$items = array_filter( $response['data'], array( $this, 'checkMetadata' ) );
		foreach ( $items as $item ) {
			$advice = $item['advice'];

			$advice['created_at'] = $item['created_at'];
			$advice['updated_at'] = $item['updated_at'];

			if ( ! $this->checkAdvice( $advice ) ) {
				continue;
			}

			$this->setRequests( $advice );
			$this->set( $advice );
		}
	}

	/**
	 * Set requests.
	 *
	 * @param array $advice advice.
	 *
	 * @return void
	 */
	private function setRequests( &$advice ) {
		$json = $this->api()->details( $advice['id'] );
		if ( ! empty( $json ) ) {
			$details = json_decode( $json, true );
			if (
				is_array( $details ) &&
				array_key_exists( 'data', $details ) &&
				array_key_exists( 'advice', $details['data'] ) &&
				array_key_exists( 'requests', $details['data']['advice'] ) &&
				is_array( $details['data']['advice']['requests'] )
			) {
				$advice['requests'] = array();
				foreach ( $details['data']['advice']['requests'] as $request ) {
					$advice['requests'][ $request['id'] ] = $request['url'];
				}
			}
		}
	}

	/**
	 * Sync statuses.
	 *
	 * @param ?int $advice_id id.
	 *
	 * @return void
	 */
	public function syncStatuses( $advice_id = 0 ) {
		if ( (int) $advice_id > 0 ) {
			$this->status( $advice_id, true );
		} else {
			$advices = $this->advices();

			foreach ( $advices as $advice ) {
				$this->status( $advice->id, false );
			}

			$this->save();
		}
	}

	/**
	 * Advice modifier.
	 *
	 * @param int      $advice_id advice.
	 * @param callable $callback command.
	 * @param bool     $save advice.
	 * @param string   $reason advice rollback reason.
	 *
	 * @return void
	 */
	private function modifier( $advice_id, $callback, $save = true, $reason = '' ) {
		$advices = $this->advices();

		if ( ! array_key_exists( $advice_id, $advices ) ) {
			do_action( 'cl_smart_advice_notice_add', $advice_id, NoticeModel::ERROR_ADVICE_NOT_FOUND['type'], array( 'advice_id' => $advice_id ) );

			return;
		}

		$advice = $advices[ $advice_id ];

		if ( is_callable( $callback ) ) {
			$callback( $advice, $reason );
		}

		$this->set( $advice->toArray() );

		if ( true === $save ) {
			$this->save();
		}
	}

	/**
	 * Apply advice, set status pending.
	 *
	 * @param int  $advice_id to apply.
	 * @param bool $save advices.
	 *
	 * @return void
	 */
	public function apply( $advice_id, $save = true ) {
		$this->modifier(
			$advice_id,
			function ( $advice ) {
				$json = $this->api()->apply( $advice->id );
				if ( ! $json ) {
					do_action( 'cl_smart_advice_set_error', E_USER_WARNING, 'Advice manager: failed modifier apply for ' . $advice->id, __FILE__, __LINE__ );
					do_action( 'cl_smart_advice_notice_add', $advice->id, NoticeModel::ERROR_ADVICE_APPLY['type'], array( 'advice_type' => $advice->type ) );
				} else {
					$response = json_decode( $json, true );
					$this->errorHandler( $response, $advice, NoticeModel::ERROR_ADVICE_APPLY['type'] );

					$advice->status = 'pending';
				}
			},
			$save
		);
	}

	/**
	 * Apply advice by type, set status pending.
	 *
	 * @param string $type to apply.
	 * @param bool   $save advices.
	 *
	 * @return void
	 */
	public function applyType( $type, $save = true ) {
		$advice = current(
			array_filter(
				$this->advices(),
				function ( $advice ) use ( $type ) {
					return strtolower( $advice->type ) === strtolower( $type );
				}
			)
		);

		if ( $advice ) {
			$this->apply( $advice->id, $save );
		}
	}

	/**
	 * Rollback advice, set status pending.
	 *
	 * @param int    $advice_id id.
	 * @param bool   $save advices.
	 * @param string $reason advice rollback reason.
	 *
	 * @return void
	 */
	public function rollback( $advice_id, $save = true, $reason = '' ) {
		$this->modifier(
			$advice_id,
			function ( $advice, $reason ) {
				$json = $this->api()->rollback( $advice->id, $reason );
				if ( ! $json ) {
					do_action( 'cl_smart_advice_set_error', E_USER_WARNING, 'Advice manager: failed modifier rollback for ' . $advice->id, __FILE__, __LINE__ );
					do_action( 'cl_smart_advice_notice_add', $advice->id, NoticeModel::ERROR_ADVICE_ROLLBACK['type'], array( 'advice_type' => $advice->type ) );
				} else {
					$response = json_decode( $json, true );
					$this->errorHandler( $response, $advice, NoticeModel::ERROR_ADVICE_ROLLBACK['type'] );

					$advice->status = 'pending';
				}
			},
			$save,
			$reason
		);
	}

	/**
	 * Subscription advice payment status.
	 *
	 * @param int $advice_id id.
	 *
	 * @return void
	 */
	public function subscription( $advice_id ) {
		$this->api()->subscription( $advice_id );
	}

	/**
	 * Update advice status.
	 *
	 * @param int  $advice_id id.
	 * @param bool $save advices.
	 *
	 * @return void
	 */
	public function status( $advice_id, $save = true ) {
		$this->modifier(
			$advice_id,
			function ( $advice ) {
				$json = $this->api()->status( $advice->id );

				$response = json_decode( $json, true );
				if ( is_array( $response ) ) {
					if ( array_key_exists( 'status', $response ) ) {
						$advice->status = $response['status'];
					}

					if ( array_key_exists( 'total_stages', $response ) ) {
						$advice->total_stages = $response['total_stages'];
					}

					if ( array_key_exists( 'completed_stages', $response ) ) {
						$advice->completed_stages = $response['completed_stages'];
					}

					if ( array_key_exists( 'subscription', $response ) ) {
						$module_name = ! empty( $advice->module_name ) ? $advice->module_name : strtolower( $advice->type );
						if ( array_key_exists( $module_name, $response['subscription'] ) ) {
							$advice->subscription_status = $response['subscription'][ $module_name ];
						}
					}

					if ( array_key_exists( 'upgrade_url', $response ) ) {
						$advice->subscription_upgrade_url = $response['upgrade_url'];
					}

					$this->errorHandler( $response, $advice, NoticeModel::ERROR_ADVICE_STATUS['type'] );
				}

				return $json;
			},
			$save
		);
	}

	/**
	 * Error handler.
	 *
	 * @param array  $response data.
	 * @param Model  $advice advice.
	 * @param string $error_type type.
	 *
	 * @return void
	 */
	private function errorHandler( $response, $advice, $error_type ) {
		if ( ! is_array( $response ) ) {
			return;
		}

		$errors = array();

		// Result.
		if ( array_key_exists( 'result', $response ) && 'success' !== $response['result'] ) {
			$context     = array();
			$description = $response['result'];

			if ( array_key_exists( 'context', $response ) && is_array( $response['context'] ) ) {
				$context = $response['context'];
			}

			$errors[] = $this->errorContext( $description, $context );
		}

		// Features.
		$this->errorFeature( $response, $errors );

		// Data result.
		if ( array_key_exists( 'data', $response ) && array_key_exists( 'result', $response['data'] ) && 'success' !== $response['data']['result'] ) {
			$context     = array();
			$description = $response['data']['result'];

			if ( 'PAYMENT_REQUIRED' !== $description ) {
				if ( array_key_exists( 'context', $response['data'] ) && is_array( $response['data']['context'] ) ) {
					$context = $response['data']['context'];
				}

				$errors[] = $this->errorContext( $description, $context );

				// Data features.
				if ( array_key_exists( 'feature', $response['data'] ) ) {
					$this->errorFeature( $response['data'], $errors );
				}
			}
		}

		// Warning.
		if ( array_key_exists( 'warning', $response ) && ! empty( $response['warning'] ) ) {
			$context     = array();
			$description = $response['warning'];

			if ( array_key_exists( 'context', $response ) && is_array( $response['context'] ) ) {
				$context = $response['context'];
			}

			$errors[] = $this->errorContext( $description, $context );
		}

		$error = implode( PHP_EOL, $errors );

		if ( ! empty( $error ) ) {
			do_action(
				'cl_smart_advice_notice_add',
				$advice->id,
				$error_type,
				array(
					'advice_type' => $advice->type,
					'description' => $error,
				)
			);
		}
	}

	/**
	 * Find error features.
	 *
	 * @param array $response data.
	 * @param array $errors output.
	 *
	 * @return void
	 */
	private function errorFeature( $response, &$errors ) {
		if ( array_key_exists( 'feature', $response ) && array_key_exists( 'issues', $response['feature'] ) && ! empty( $response['feature']['issues'] ) ) {
			foreach ( $response['feature']['issues'] as $issue ) {
				$context      = array();
				$descriptions = array();

				if ( array_key_exists( 'header', $issue ) ) {
					$descriptions[] = $issue['header'];
				}

				if ( array_key_exists( 'description', $issue ) ) {
					// Too long to show.
					if ( strpos( $issue['description'], 'wp-cli' ) === false ) {
						$descriptions[] = PHP_EOL . $issue['description'];
					}
				}

				if ( array_key_exists( 'fix_tip', $issue ) ) {
					$descriptions[] = PHP_EOL . $issue['fix_tip'];
				}

				if ( array_key_exists( 'context', $issue ) && is_array( $issue['context'] ) ) {
					$context = $issue['context'];
				}

				$description = implode( PHP_EOL, $descriptions );

				$errors[] = $this->errorContext( $description, $context );
			}
		}
	}

	/**
	 * Error parse.
	 *
	 * @param string $description error string.
	 * @param array  $context context.
	 *
	 * @return string
	 */
	private function errorContext( $description, $context = array() ) {
		$search  = array();
		$replace = array();
		foreach ( $context as $key => $val ) {
			$search[]  = '%(' . $key . ')s';
			$replace[] = $val;
		}

		return str_replace( $search, $replace, $description );
	}

	/**
	 * All cached advices.
	 *
	 * @return array
	 */
	private function cache() {
		$advices = $this->option->get( 'advices' );

		return array_map(
			function ( $advice ) {
				return ( new Model() )->fill( $advice );
			},
			$advices
		);
	}

	/**
	 * Time sync at.
	 *
	 * @return int|null
	 */
	public function syncAt() {
		return $this->option->get( 'sync_at' );
	}

	/**
	 * Update advices' cache.
	 *
	 * @return void
	 */
	public function save() {
		$advices = $this->advices();

		$this->option->save( 'advices', $advices );
		$this->option->save( 'sync_at', time() );
	}

	/**
	 * Notify if advice is new
	 *
	 * @return void
	 */
	private function notifyNew() {
		$current = array_map(
			function ( $advice ) {
				return $advice->id;
			},
			$this->cache()
		);

		$new = array_filter(
			$this->advices(),
			function ( $advice ) use ( $current ) {
				return ! in_array( $advice->id, $current );
			}
		);

		$count = count( $new );

		if ( $count > 0 ) {
			$type = NoticeModel::NEW_ADVICES['type'];
			do_action( 'cl_smart_advice_notice_add', $type, $type, array( 'count' => $count ) );
			do_action( 'cl_smart_advice_email', 'advices', array( 'advice_ids' => array_column( $new, 'id' ) ) );
		}
	}

	/**
	 * Has pending advices.
	 *
	 * @return bool
	 */
	public function hasPending() {
		$advices = $this->advices();
		$pending = array_filter(
			$advices,
			function ( $advice ) {
				return 'pending' === $advice->statusUi();
			}
		);

		return count( $pending ) > 0;
	}

	/**
	 * Need subscription.
	 *
	 * @param int $advice_id id.
	 *
	 * @return false|string
	 */
	public function subscriptionType( $advice_id ) {
		$advices = $this->advices();
		if ( array_key_exists( $advice_id, $advices ) ) {
			return $advices[ $advice_id ]->subscriptionType();
		}

		return false;
	}

	/**
	 * Need agreement.
	 *
	 * @param int $advice_id id.
	 *
	 * @return false|string
	 */
	public function agreementType( $advice_id ) {
		$advices = $this->advices();
		if ( array_key_exists( $advice_id, $advices ) ) {
			return $advices[ $advice_id ]->agreementType();
		}

		return false;
	}

	/**
	 * Plugin compatible.
	 *
	 * @return bool
	 */
	public function compatible() {
		return function_exists( 'stream_socket_client' );
	}

	/**
	 * Agreement text.
	 *
	 * @param string $type advice type string, e.g. 'cdn'.
	 *
	 * @return string
	 */
	public function agreementText( $type ) {
		$json = $this->api()->agreement( $type );
		if ( ! empty( $json ) ) {
			$data = json_decode( $json, true );
			if ( is_array( $data ) && array_key_exists( 'text', $data ) ) {
				return $data['text'];
			}
		}

		return '';
	}
}
