<?php
/**
 * Copyright (с) Cloud Linux GmbH & Cloud Linux Software, Inc 2010-2022 All Rights Reserved
 *
 * Licensed under CLOUD LINUX LICENSE AGREEMENT
 * https://www.cloudlinux.com/legal/
 */

namespace CloudLinux\SmartAdvice\App\Service;

/**
 * Mail advices
 */
class MailerAdvices extends Mailer {

	/**
	 * Analytics service.
	 *
	 * @var Analytics
	 */
	public $analytics;

	/**
	 * Number of new advices.
	 *
	 * @var int
	 */
	private $count = 0;

	/**
	 * Advice ids.
	 *
	 * @var array
	 */
	private $advice_ids = array();

	/**
	 * Constructor.
	 *
	 * @param array $payload data.
	 */
	public function __construct( $payload ) {
		if ( array_key_exists( 'advice_ids', $payload ) ) {
			$this->count      = count( $payload['advice_ids'] );
			$this->advice_ids = $payload['advice_ids'];
		}
	}

	/**
	 * Can send to WP users.
	 *
	 * @return bool
	 */
	public function availableToWpUsers() {
		return $this->count > 0;
	}

	/**
	 * Can send to Panel users.
	 *
	 * @return bool
	 */
	public function availableToPanelUsers() {
		return $this->count > 0;
	}

	/**
	 * Subject.
	 *
	 * @param string $site_url url.
	 *
	 * @return string
	 */
	public function subject( $site_url ) {
		$subject = 'New performance Advice';
		if ( ! empty( $site_url ) ) {
			$subject .= ' for ' . $site_url;
		}

		return $subject;
	}

	/**
	 * Template file.
	 *
	 * @return string
	 */
	public function template() {
		return 'letter.advices';
	}

	/**
	 * Template data.
	 *
	 * @return array
	 */
	public function payload() {
		return array(
			'number_of_advices' => $this->count,
		);
	}

	/**
	 * {@inheritDoc}
	 */
	public function filter_template_data( $email, $data ) {
		if ( ! is_array( $data ) ) {
			return $data;
		}

		// Replace the original link to advice UI with redirect via Smart Advice analytics service.
		if ( array_key_exists( 'panel_link', $data ) ) {
			$data['panel_link'] = $this->analytics->get_redirect_link(
				$email,
				$this->advice_ids,
				'email_link_clicked',
				$data['panel_link']
			);
		}

		// Add analytics pixel to the footer.
		$footer         = array_key_exists( 'footer', $data ) ? $data['footer'] : '';
		$data['footer'] = $footer . sprintf(
			'<img src="%s" width="1" height="1" />',
			$this->analytics->get_pixel_url( $email, $this->advice_ids, 'email_opened' )
		);

		return $data;
	}

	/**
	 * {@inheritDoc}
	 */
	public function post_email_sent( $success, $user_type, $email, $subject, $body, $headers ) {
		$this->analytics->send_event( $email, $this->advice_ids, $success ? 'email_sent' : 'email_error' );
	}

	/**
	 * {@inheritDoc}
	 */
	public function init_analytics( $salt ) {

		if ( empty( $salt ) ) {
			$salt = time();
		}

		$this->analytics = new Analytics( $salt );
	}
}
